import { FormGroupDirective, NgForm, NG_VALUE_ACCESSOR, NG_VALIDATORS, NgControl } from '@angular/forms';
import { InjectionToken, Inject, Injectable, LOCALE_ID, Optional, Directive, ElementRef, Input, Attribute, Output, EventEmitter, TemplateRef, ChangeDetectorRef, ContentChildren, ViewContainerRef, HostListener, Renderer2, Self, HostBinding, Pipe, NgModule, defineInjectable } from '@angular/core';
import { DatePipe } from '@angular/common';
import { DragController, DropController, LocalStorage, SessionStorage } from '@anjuna/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TENANT_TIMEZONE_STRING = new InjectionToken('TenantTimezoneString');
/** @type {?} */
const TENANT_MERIDIEM = new InjectionToken('TenantMeridiem');
/** @type {?} */
const TIMEZONE_OVERRIDE = new InjectionToken('TimezoneOverride');
/** @type {?} */
const MOMENT = new InjectionToken('MomentJs');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* Note: Typings of 'any' in this file refer to Moment.
         The Moment type does not come without the full weight of Moment.js, so it was intentionally left out. */
// Adapter for Date-Picker to accept String, Native Date or Moment objects
class MomentDateAdapter {
    /**
     * @param {?} _moment
     * @param {?=} _locale
     * @param {?=} _timezone
     */
    constructor(_moment, _locale, _timezone) {
        this._moment = _moment;
        this._locale = _locale;
        this._timezone = _timezone;
        this.moment = _moment;
        this.timezone = _timezone;
        if (this.moment) {
            this.moment.locale(this._locale);
            if (this.timezone) {
                this.moment.tz.setDefault(this.timezone);
            }
            else {
                console.warn('Timezone is not set, is defaulting to system timezone. TIMEZONE_OVERRIDE may be set at the Module level.');
            }
        }
        else {
            console.error('Package "moment" has not been installed. Please either install "moment" or use the NativeDateAdapter.');
        }
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    dayOfFirst(year, month) {
        /** @type {?} */
        const adjusted = this.handleMonthAdjustment(year, month);
        return this.moment([adjusted.year, adjusted.month, 1]).day();
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    daysInMonth(year, month) {
        /** @type {?} */
        const adjusted = this.handleMonthAdjustment(year, month);
        return this.moment([adjusted.year, adjusted.month]).daysInMonth();
    }
    /**
     * @param {?} evalDate
     * @param {?} dateRangeParams
     * @return {?}
     */
    isDisabled(evalDate, dateRangeParams) {
        if (!dateRangeParams || (!dateRangeParams.minDate && !dateRangeParams.maxDate)) {
            return false;
        }
        /** @type {?} */
        const dayIsBeforeMinDate = this.moment(evalDate).isBefore(this.moment(dateRangeParams.minDate), 'day');
        /** @type {?} */
        const dayIsAfterMaxDate = this.moment(evalDate).isAfter(this.moment(dateRangeParams.maxDate), 'day');
        return dayIsBeforeMinDate || dayIsAfterMaxDate;
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    getMonthInfo(year, month) {
        /** @type {?} */
        const adjusted = this.handleMonthAdjustment(year, month);
        return new Date(this.moment([adjusted.year, adjusted.month, 15]));
    }
    /**
     * @param {?} date
     * @return {?}
     */
    month(date) {
        return this.moment(date).month();
    }
    /**
     * @param {?=} year
     * @param {?=} month
     * @param {?=} day
     * @return {?}
     */
    newDateFromArray(year, month = 0, day = 1) {
        return new Date(this.moment([year, month, day]));
    }
    /**
     * @param {?=} value
     * @return {?}
     */
    newDateFromValue(value) {
        if (!value) {
            return new Date();
        }
        else if (value instanceof Date) {
            return new Date(this.moment(value));
        }
        else {
            return new Date(value);
        }
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderDate(date) {
        return this.moment(date).locale(this.formatterLanguage || this._locale).format('ddd, MMM D');
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderYear(date) {
        return this.moment(date).locale(this.formatterLanguage || this._locale).format('YYYY');
    }
    /**
     * @param {?} _calendarDay
     * @param {?} _comparator
     * @return {?}
     */
    sameDay(_calendarDay, _comparator) {
        /** @type {?} */
        const calendarDay = this.moment([_calendarDay.year, _calendarDay.month, _calendarDay.day]);
        /** @type {?} */
        const comparator = this.moment(_comparator);
        return calendarDay.isSame(comparator, 'day');
    }
    /**
     * @param {?} date
     * @return {?}
     */
    year(date) {
        return this.moment(date).year();
    }
    /**
     * @return {?}
     */
    weekdays() {
        return this.moment.weekdays();
    }
    /**
     * @return {?}
     */
    months() {
        return this.moment.monthsShort();
    }
    /**
     * @private
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    handleMonthAdjustment(year, month) {
        if (month > 11) {
            month = month - 12;
            year++;
        }
        else if (month < 0) {
            month = month + 12;
            year--;
        }
        return { year, month };
    }
    /**
     * @param {?} input
     * @return {?}
     */
    prepDateStringOutput(input) {
        return (input) ? this.moment(this.processDateInput(input)).format('l') : '';
    }
    /**
     * @param {?} input
     * @return {?}
     */
    processDateInput(input) {
        if (!input) {
            return null;
        }
        if (input && this.moment(input, 'L', this._locale).isValid()) {
            /** @type {?} */
            const timezone = this.timezone || this.moment.tz.guess();
            return this.moment(input, 'L', this._locale).tz(timezone).startOf('day').toDate();
        }
        return new Date('false');
    }
}
MomentDateAdapter.decorators = [
    { type: Injectable }
];
/** @nocollapse */
MomentDateAdapter.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [MOMENT,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TIMEZONE_OVERRIDE,] }] }
];
/**
 * @param {?} MOMENT_
 * @param {?=} locale
 * @param {?=} timezone
 * @return {?}
 */
function getMomentDateAdapter(MOMENT_, locale, timezone) {
    return new MomentDateAdapter(MOMENT_, locale, timezone);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const STANDARD_LOCALES = {
    'en-US': 'M/d/yyyy',
    'sw-KE': 'M/d/yyyy',
    'en-ZW': 'M/d/yyyy',
    'en-PH': 'M/d/yyyy',
    'rw-RW': 'M/d/yyyy',
    'moh-CA': 'M/d/yyyy',
    'fil-PH': 'M/d/yyyy',
    'ne-NP': 'M/d/yyyy',
    'es-US': 'M/d/yyyy',
    'sq-AL': 'yyyy/M/d',
    'bg-BG': 'yyyy/M/d',
    'sr-BA': 'yyyy/M/d',
    'fr-CA': 'yyyy/M/d',
    'zh-CN': 'yyyy/M/d',
    'zh-HK': 'yyyy/M/d',
    'hu-HU': 'yyyy/M/d',
    'ja-JP': 'yyyy/M/d',
    'ko-KR': 'yyyy/M/d',
    'lt-LT': 'yyyy/M/d',
    'sv-SE': 'yyyy/M/d',
    'zh-TW': 'yyyy/M/d',
    'en-ZA': 'yyyy/M/d'
};
/**
 * @abstract
 * @template T
 */
class DateAdapter {
}
// Default adapter, will accept String or Native Date
class NativeDateAdapter {
    /**
     * @param {?=} _locale
     */
    constructor(_locale) {
        this._locale = _locale;
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    dayOfFirst(year, month) {
        return new Date(year, month, 1).getDay();
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    daysInMonth(year, month) {
        return new Date(year, month + 1, 0).getDate();
    }
    /**
     * @param {?} evalDate
     * @param {?} dateRangeParams
     * @return {?}
     */
    isDisabled(evalDate, dateRangeParams) {
        if (!dateRangeParams) {
            return false;
        }
        /** @type {?} */
        const dayIsBeforeMinDate = (dateRangeParams.minDate) ? evalDate < dateRangeParams.minDate : false;
        /** @type {?} */
        const dayIsAfterMaxDate = (dateRangeParams.maxDate) ? evalDate > dateRangeParams.maxDate : false;
        return dayIsBeforeMinDate || dayIsAfterMaxDate;
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    getMonthInfo(year, month) {
        return new Date(year, month, 15);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    month(date) {
        return date.getMonth();
    }
    /**
     * @param {?=} year
     * @param {?=} month
     * @param {?=} day
     * @return {?}
     */
    newDateFromArray(year, month, day) {
        return new Date(year, month, day);
    }
    /**
     * @param {?=} value
     * @return {?}
     */
    newDateFromValue(value) {
        return (value) ? new Date(value) : new Date();
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderDate(date) {
        return localize(this.languageCode, date, 'EEE, MMM d');
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderYear(date) {
        return localize(this.languageCode, date, 'yyyy');
    }
    /**
     * @param {?} _calendarDay
     * @param {?} _comparator
     * @return {?}
     */
    sameDay(_calendarDay, _comparator) {
        return _calendarDay.day === _comparator.getDate()
            && _calendarDay.month === _comparator.getMonth()
            && _calendarDay.year === _comparator.getFullYear();
    }
    /**
     * @param {?} date
     * @return {?}
     */
    year(date) {
        return date.getFullYear();
    }
    /**
     * @return {?}
     */
    weekdays() {
        return getWeekdays(this.languageCode);
    }
    /**
     * @return {?}
     */
    months() {
        return getMonths(this.languageCode);
    }
    /**
     * @param {?} input
     * @return {?}
     */
    prepDateStringOutput(input) {
        return (input) ? new Date(this.processDateInput(input)).toLocaleDateString(this._locale) : '';
    }
    /**
     * @param {?} input
     * @return {?}
     */
    processDateInput(input) {
        if (!input) {
            return null;
        }
        if (typeof input === 'string') {
            /** @type {?} */
            const dateSegments = parseInput(input, this._locale);
            if (Array.isArray(dateSegments.splitDate) && dateSegments.splitDate.length === 3 && dateSegments.year) {
                return new Date(parseInt(dateSegments.year, 10), parseInt(dateSegments.month, 10) - 1, parseInt(dateSegments.day, 10));
            }
            else {
                return new Date(input);
            }
        }
        else {
            return input;
        }
    }
}
/** @nocollapse */
NativeDateAdapter.ctorParameters = () => [
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
];
/**
 * @param {?} lang
 * @return {?}
 */
function getWeekdays(lang) {
    /** @type {?} */
    const weekdays = [];
    for (let i = 0; i < 7; i++) {
        /** @type {?} */
        const date = new Date();
        /** @type {?} */
        const testDate = date.setDate(date.getDate() - date.getDay() + i);
        weekdays.push(localize(lang, new Date(testDate), 'EEE'));
    }
    return weekdays;
}
/**
 * @param {?} lang
 * @return {?}
 */
function getMonths(lang) {
    /** @type {?} */
    const months = [];
    for (let i = 0; i < 12; i++) {
        /** @type {?} */
        const testDate = new Date().setMonth(i);
        months.push(localize(lang, new Date(testDate), 'MMM'));
    }
    return months;
}
/**
 * @param {?=} _languageCode
 * @param {?=} date
 * @param {?=} format
 * @return {?}
 */
function localize(_languageCode = 'en', date, format) {
    /** @type {?} */
    const pipe = new DatePipe(_languageCode);
    return pipe.transform(date, format);
}
/* Currently only support shortDate strings (with non-alphanumeric separators) of:
    {
        Month-Day-Year: if included in STANDARD_LOCALES,
        Year-Month-Day: if included in STANDARD_LOCALES,
        Day-Month-Year: for all locales outside of STANDARD_LOCALES
    }
 */
/**
 * @param {?} input
 * @param {?} locale
 * @return {?}
 */
function parseInput(input, locale) {
    /** @type {?} */
    let format = STANDARD_LOCALES[locale];
    // Covers case of partial locale code provided
    if (!format) {
        /** @type {?} */
        const localeKey = Object.keys(STANDARD_LOCALES).find(key => key.includes(locale));
        format = STANDARD_LOCALES[localeKey];
    }
    // Split string on any non-alphanumeric characters, not including spaces
    /** @type {?} */
    const regexSplit = input.split(/[^\w\s]+/g).filter(Boolean);
    // Parse digit-only string, or use the regexSplit value
    /** @type {?} */
    const splitDate = (regexSplit.length <= 1 && input.length === 8) ? parseDigitOnlyDate(input, format) : regexSplit;
    /** @type {?} */
    const dateSegments = {
        splitDate,
        year: '',
        month: '',
        day: ''
    };
    /** @type {?} */
    const splitDateCp = splitDate.slice(0);
    // Set year
    dateSegments.year = setYearSegment(splitDateCp);
    // Set Day and Month
    Object.assign(dateSegments, setMonthAndDaySegments(splitDateCp, format));
    return dateSegments;
}
/**
 * @param {?} splitDateArray
 * @return {?}
 */
function setYearSegment(splitDateArray) {
    // Find year
    /** @type {?} */
    const year = splitDateArray.find(segment => segment.length === 4);
    // Remove year from working array
    /** @type {?} */
    const yearIndex = splitDateArray.indexOf(year);
    splitDateArray.splice(yearIndex, 1);
    return year;
}
/**
 * @param {?} splitDateArray
 * @param {?} format
 * @return {?}
 */
function setMonthAndDaySegments(splitDateArray, format) {
    /** @type {?} */
    const segments = {
        day: '',
        month: ''
    };
    // See if we can determine which segment is the days segment
    /** @type {?} */
    const assumedDay = splitDateArray.find(segment => parseInt(segment, 10) > 12);
    if (assumedDay) {
        segments.day = assumedDay;
        /** @type {?} */
        const dayIndex = splitDateArray.indexOf(assumedDay);
        splitDateArray.splice(dayIndex, 1);
        segments.month = splitDateArray[0];
        // Otherwise, make a reasonable guess
    }
    else {
        switch (format) {
            case 'M/d/yyyy':
            case 'yyyy/M/d':
                segments.month = splitDateArray[0];
                segments.day = splitDateArray[1];
                break;
            // Assume 'dd/MM/yyyy' for anything else
            default:
                segments.month = splitDateArray[1];
                segments.day = splitDateArray[0];
                break;
        }
    }
    return segments;
}
/**
 * @param {?} input
 * @param {?} format
 * @return {?}
 */
function parseDigitOnlyDate(input, format) {
    /** @type {?} */
    let splitDate;
    switch (format) {
        case 'yyyy/M/d':
            splitDate = [
                input.slice(0, 4),
                input.slice(4, 6),
                input.slice(6)
            ];
            break;
        // Assume 'dd/MM/yyyy' or 'M/d/yyyy' for anything else
        default:
            splitDate = [
                input.slice(0, 2),
                input.slice(2, 4),
                input.slice(4)
            ];
            break;
    }
    return splitDate;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Adapter for UTC dates
class UTCDateAdapter {
    /**
     * @param {?=} _locale
     */
    constructor(_locale) {
        this._locale = _locale;
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    dayOfFirst(year, month) {
        return new Date(year, month, 1).getDay();
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    daysInMonth(year, month) {
        return new Date(year, month + 1, 0).getDate();
    }
    /**
     * @param {?} evalDate
     * @param {?} dateRangeParams
     * @return {?}
     */
    isDisabled(evalDate, dateRangeParams) {
        if (!dateRangeParams) {
            return false;
        }
        /** @type {?} */
        const dayIsBeforeMinDate = (dateRangeParams.minDate) ? evalDate < dateRangeParams.minDate : false;
        /** @type {?} */
        const dayIsAfterMaxDate = (dateRangeParams.maxDate) ? evalDate > dateRangeParams.maxDate : false;
        return dayIsBeforeMinDate || dayIsAfterMaxDate;
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    getMonthInfo(year, month) {
        return new Date(year, month, 15);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    month(date) {
        return date.getUTCMonth();
    }
    /**
     * @param {?=} year
     * @param {?=} month
     * @param {?=} day
     * @return {?}
     */
    newDateFromArray(year, month, day) {
        return new Date(Date.UTC(year, month, day));
    }
    /**
     * @param {?=} value
     * @return {?}
     */
    newDateFromValue(value) {
        return (value) ? this.processDateInput(value) : this.getZerodDate(new Date());
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderDate(date) {
        return localize(this.languageCode, this.addTimezoneOffset(date), 'EEE, MMM d');
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderYear(date) {
        return localize(this.languageCode, this.addTimezoneOffset(date), 'yyyy');
    }
    /**
     * @param {?} _calendarDay
     * @param {?} _comparator
     * @return {?}
     */
    sameDay(_calendarDay, _comparator) {
        return _calendarDay.day === _comparator.getUTCDate()
            && _calendarDay.month === _comparator.getUTCMonth()
            && _calendarDay.year === _comparator.getUTCFullYear();
    }
    /**
     * @param {?} date
     * @return {?}
     */
    year(date) {
        return date.getUTCFullYear();
    }
    /**
     * @return {?}
     */
    weekdays() {
        return getWeekdays(this.languageCode);
    }
    /**
     * @return {?}
     */
    months() {
        return getMonths(this.languageCode);
    }
    /**
     * @param {?} input
     * @return {?}
     */
    prepDateStringOutput(input) {
        return (input) ? new Date(this.processDateInput(input)).toLocaleDateString(this._locale, { timeZone: 'UTC' }) : '';
    }
    /**
     * @param {?} input
     * @return {?}
     */
    processDateInput(input) {
        if (!input) {
            return null;
        }
        if (typeof input === 'string') {
            /** @type {?} */
            const dateSegments = parseInput(input, this._locale);
            if (Array.isArray(dateSegments.splitDate) && dateSegments.splitDate.length === 3 && dateSegments.year) {
                return new Date(Date.UTC(parseInt(dateSegments.year, 10), parseInt(dateSegments.month, 10) - 1, parseInt(dateSegments.day, 10)));
            }
            else {
                // This is a fallback for malformed date string, outcome unknown
                return new Date(input);
            }
        }
        else {
            return this.getZerodDate(input);
        }
    }
    /**
     * @private
     * @param {?} date
     * @return {?}
     */
    addTimezoneOffset(date) {
        /** @type {?} */
        const newDate = new Date(date);
        /** @type {?} */
        const timeOffsetInMS = date.getTimezoneOffset() * 60000;
        newDate.setTime(date.getTime() + timeOffsetInMS);
        return newDate;
    }
    /**
     * @private
     * @param {?} date
     * @return {?}
     */
    getZerodDate(date) {
        /** @type {?} */
        const safeDate = (date._isAMomentObject) ? new Date(date) : date;
        /** @type {?} */
        const day = safeDate.getUTCDate();
        /** @type {?} */
        const month = safeDate.getUTCMonth();
        /** @type {?} */
        const year = safeDate.getUTCFullYear();
        return new Date(Date.UTC(year, month, day));
    }
}
UTCDateAdapter.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UTCDateAdapter.ctorParameters = () => [
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drag API.
 */
class DraggableDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
        this.dataFn = () => this.getData();
        this.controller = new DragController(this.element.nativeElement, this.dataFn).connect();
    }
    /**
     * @private
     * @return {?}
     */
    getData() {
        return this.anjDraggable;
    }
}
DraggableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjDraggable]'
            },] }
];
/** @nocollapse */
DraggableDirective.ctorParameters = () => [
    { type: ElementRef }
];
DraggableDirective.propDecorators = {
    anjDraggable: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drop API.
 */
class DroppableDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
        this.controller = new DropController(this.element.nativeElement).connect();
    }
}
DroppableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjDroppable]'
            },] }
];
/** @nocollapse */
DroppableDirective.ctorParameters = () => [
    { type: ElementRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@internal
 */
class ResizerDirective {
    /**
     * @param {?} anjResizer
     * @param {?} element
     */
    constructor(anjResizer, element) {
        this.anjResizer = anjResizer;
        this.element = element;
        this.resize = new EventEmitter();
        this.resizeListener = this.handleResize.bind(this);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        window.addEventListener('resize', this.resizeListener);
        this.parentElement = this.element.nativeElement.parentElement;
        this.getDimensions();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        window.clearTimeout(this.resizeDebouncer);
        window.removeEventListener('resize', this.resizeListener);
    }
    /**
     * @return {?}
     */
    handleResize() {
        window.clearTimeout(this.resizeDebouncer);
        this.resizeDebouncer = window.setTimeout(() => {
            /** @type {?} */
            const needsResize = this.getDimensions();
            if (needsResize) {
                this.resize.emit();
            }
        }, this.anjResizer);
    }
    /**
     * @return {?}
     */
    getDimensions() {
        /** @type {?} */
        const parentWidth = this.parentElement.clientWidth;
        /** @type {?} */
        const parentHeight = this.parentElement.clientHeight;
        if (parentWidth === this.parentWidth && parentHeight === this.parentHeight) {
            return false;
        }
        this.parentWidth = parentWidth;
        this.parentHeight = parentHeight;
        return true;
    }
}
ResizerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjResizer]'
            },] }
];
/** @nocollapse */
ResizerDirective.ctorParameters = () => [
    { type: Number, decorators: [{ type: Attribute, args: ['anjResizer',] }] },
    { type: ElementRef }
];
ResizerDirective.propDecorators = {
    resize: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DomOutlet {
    /**
     * @param {?} targetElement
     */
    constructor(targetElement) {
        this.targetElement = targetElement;
    }
    /**
     * @return {?}
     */
    get viewRef() {
        return this._viewRef;
    }
    /**
     * @template C
     * @param {?} template
     * @return {?}
     */
    attachTemplate(template) {
        this._viewRef = template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
        this._viewRef.detectChanges();
        this._viewRef.rootNodes.forEach(rootNode => this.targetElement.appendChild(rootNode));
        this.setDisposeFn((() => {
            /** @type {?} */
            const index = template.viewContainerRef.indexOf(this._viewRef);
            if (index !== -1) {
                template.viewContainerRef.remove(index);
            }
        }));
        return this._viewRef;
    }
    /**
     * @param {?} context
     * @return {?}
     */
    updateContext(context) {
        for (const prop in context) {
            if (context.hasOwnProperty(prop)) {
                this._viewRef.context[prop] = context[prop];
            }
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    setDisposeFn(fn) {
        this.disposeFn = fn;
    }
    /**
     * @param {?} template
     * @return {?}
     */
    attach(template) {
        this.template = template;
        return this.attachTemplate(template);
    }
    /**
     * @return {?}
     */
    detach() {
        if (this.template) {
            this.template.setAttachedHost(null);
            this.template = null;
        }
        this.invokeDisposeFn();
    }
    /**
     * @return {?}
     */
    dispose() {
        this.disposeFn();
        if (this.targetElement.parentNode != null) {
            this.targetElement.parentNode.removeChild(this.targetElement);
        }
        return undefined;
    }
    /**
     * @private
     * @return {?}
     */
    invokeDisposeFn() {
        if (this.disposeFn) {
            this.disposeFn();
            this.disposeFn = null;
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Template {
    /**
     * @param {?} templateRef
     * @param {?} viewContainerRef
     * @param {?=} context
     */
    constructor(templateRef, viewContainerRef, context) {
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
        this.context = context;
    }
    /**
     * @return {?}
     */
    get origin() {
        return this.templateRef.elementRef;
    }
    /**
     * @param {?} host
     * @param {?=} context
     * @return {?}
     */
    attach(host, context = this.context) {
        this.context = context;
        this.host = host;
        return host.attach(this);
    }
    /**
     * @return {?}
     */
    detach() {
        this.context = undefined;
        if (this.host) {
            /** @type {?} */
            const host = this.host;
            this.host = null;
            host.detach();
        }
    }
    /**
     * @param {?} host
     * @return {?}
     */
    setAttachedHost(host) {
        this.host = host;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TemplateDirective {
    /**
     * @param {?} _template
     */
    constructor(_template) {
        this._template = _template;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    set anjTemplateOf(name) {
        this._name = name;
    }
    /**
     * @return {?}
     */
    get name() {
        return this._name;
    }
    /**
     * @return {?}
     */
    get template() {
        return this._template;
    }
}
TemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjTemplate]'
            },] }
];
/** @nocollapse */
TemplateDirective.ctorParameters = () => [
    { type: TemplateRef }
];
TemplateDirective.propDecorators = {
    anjTemplateOf: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class AnjTable {
    /**
     * @param {?} changeRef
     * @param {?} viewRef
     * @param {?} elRef
     */
    constructor(changeRef, viewRef, elRef) {
        this.changeRef = changeRef;
        this.viewRef = viewRef;
        this.outletRows = [];
        this.element = elRef.nativeElement;
        this.element.renderer = this;
    }
    /**
     * @private
     * @param {?} row
     * @param {?} index
     * @param {?=} column
     * @return {?}
     */
    static rowContext(row, index, column) {
        return {
            $implicit: row,
            column,
            index
        };
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.outletRows.map((rowMap) => {
            for (const templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose();
                }
            }
            return rowMap;
        });
    }
    /**
     * @param {?} cell
     * @param {?} offset
     * @param {?} col
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    cellTemplate(cell, offset, col, row, rowIndex) {
        this.fromTemplate(col.cellTemplate, cell, row, rowIndex);
        col.initCell(cell, offset);
    }
    /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    fromTemplate(templateName, targetEl, row, rowIndex) {
        /** @type {?} */
        const rowMap = this.outletRows[rowIndex] || {};
        /** @type {?} */
        const outlet = rowMap[templateName];
        if (!targetEl) {
            rowMap[templateName] = outlet.dispose();
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable.rowContext(row, rowIndex));
            }
            else {
                this.updateContext(outlet, AnjTable.rowContext(row, rowIndex));
            }
        }
        else {
            rowMap[templateName] = this.createOutlet(targetEl, templateName, AnjTable.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = rowMap;
        }
    }
    /**
     * @private
     * @param {?} target
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    createOutlet(target, templateName, context) {
        /** @type {?} */
        const outlet = new DomOutlet(target);
        outlet.attach(this.createTemplate(templateName, context));
        return outlet;
    }
    /**
     * @private
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    createTemplate(templateName, context) {
        /** @type {?} */
        const templateRef = this.templates.find(t => t.name === templateName).template;
        return new Template(templateRef, this.viewRef, context);
    }
    /**
     * @private
     * @param {?} outlet
     * @param {?} context
     * @return {?}
     */
    updateContext(outlet, context) {
        outlet.updateContext(context);
        this.changeRef.markForCheck();
    }
    /**
     * @private
     * @param {?} outlet
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} context
     * @return {?}
     */
    updateOutlet(outlet, templateName, targetEl, context) {
        outlet.targetElement = targetEl;
        outlet.detach();
        outlet.attach(this.createTemplate(templateName, context));
    }
}
AnjTable.decorators = [
    { type: Directive, args: [{
                selector: 'anj-table:not([anjIgnore])',
            },] }
];
/** @nocollapse */
AnjTable.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: ViewContainerRef },
    { type: ElementRef }
];
AnjTable.propDecorators = {
    templates: [{ type: ContentChildren, args: [TemplateDirective,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const TransitionState = {
    Enter: 0,
    Leave: 1,
    EnterTo: 2,
    Inactive: 3,
};
TransitionState[TransitionState.Enter] = 'Enter';
TransitionState[TransitionState.Leave] = 'Leave';
TransitionState[TransitionState.EnterTo] = 'EnterTo';
TransitionState[TransitionState.Inactive] = 'Inactive';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@experimental
 */
class TransitionDirective {
    /**
     * @param {?} renderer
     * @param {?} element
     * @param {?} mode
     * @param {?} name
     */
    constructor(renderer, element, mode, name) {
        this.renderer = renderer;
        this.element = element;
        this.mode = mode;
        this.name = name;
        this.state = TransitionState.EnterTo;
        this.end = new EventEmitter();
        this.mode = this.mode ? this.mode : 'in-out';
        this.name = this.name ? this.name : 'transition';
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.endTransition();
        if (!changes.anjTransition.previousValue && !changes.anjTransition.currentValue && this.mode !== 'out') {
            this.initEnter();
        }
        else if (!changes.anjTransition.previousValue && changes.anjTransition.currentValue) {
            this.transitionIn();
        }
        else {
            this.transitionOut();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleTransition(event) {
        if (event.target === this.element.nativeElement && TransitionDirective.supportedTransitions.indexOf(event.propertyName) > -1) {
            event.stopPropagation();
            this.end.emit(this.state);
            if (this.state === TransitionState.Enter) {
                this.endTransition();
            }
            else {
                if (this.mode !== 'out' && this.anjTransition) {
                    this.transitionIn();
                }
                else {
                    this.endTransition();
                }
            }
        }
    }
    /**
     * @private
     * @return {?}
     */
    initEnter() {
        this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
    }
    /**
     * @private
     * @return {?}
     */
    transitionIn() {
        if (this.mode !== 'out') {
            this.state = TransitionState.Enter;
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter`);
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.endTransition();
        }
    }
    /**
     * @private
     * @return {?}
     */
    transitionOut() {
        this.state = TransitionState.Leave;
        if (this.mode !== 'in') {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave`);
        }
        else {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.state = TransitionState.EnterTo;
            if (this.anjTransition) {
                window.setTimeout(() => {
                    this.transitionIn();
                });
            }
        }
    }
    /**
     * @private
     * @return {?}
     */
    endTransition() {
        this.state = TransitionState.Inactive;
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
        if (!this.anjTransition) {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
        }
    }
}
TransitionDirective.supportedTransitions = ['opacity', 'transform'];
TransitionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjTransition]'
            },] }
];
/** @nocollapse */
TransitionDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Attribute, args: ['mode',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['name',] }] }
];
TransitionDirective.propDecorators = {
    anjTransition: [{ type: Input }],
    end: [{ type: Output }],
    handleTransition: [{ type: HostListener, args: ['transitiend', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ValueAccessor {
    /**
     * @param {?} element
     * @param {?} parentForm
     * @param {?} parentGroup
     */
    constructor(element, parentForm, parentGroup) {
        this.element = element;
        /**
         * Enable two-way binding
         */
        this.valueChange = new EventEmitter();
        this.onChange = () => { };
        this.onTouched = () => { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
        if (this.parent) {
            this.formSub = this.parent.ngSubmit.subscribe(this.checkValidity.bind(this));
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.formSub) {
            this.formSub.unsubscribe();
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.element.nativeElement.value = this.lastValue = value == null ? '' : value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onChangeEvent(value) {
        if (value !== this.lastValue) {
            this.lastValue = value;
            this.onChange(value);
            this.valueChange.emit(value);
        }
    }
    /**
     * @return {?}
     */
    onAnjBlur() {
        this.onTouched();
        this.checkValidity();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.element.nativeElement.disabled = isDisabled;
    }
    /**
     * @return {?}
     */
    checkValidity() {
        // TODO: this won't work for custom validators
        this.element.nativeElement.checkValidity();
    }
}
ValueAccessor.propDecorators = {
    valueChange: [{ type: Output }],
    onAnjBlur: [{ type: HostListener, args: ['anjBlur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class BooleanValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     */
    constructor(parentForm, parentFormGroup, element) {
        super(element, parentForm, parentFormGroup);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onAnjChange(value) {
        this.onChangeEvent(value);
        this.checkValidity();
    }
}
BooleanValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-checkbox, anj-toggle',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: BooleanValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
BooleanValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef }
];
BooleanValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target.checked'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@dynamic
 * @abstract
 */
class AnjunaValidators {
    /**
     * @param {?} c
     * @return {?}
     */
    static date(c) {
        if (!c.value) {
            return;
        }
        /** @type {?} */
        let isValid = true;
        console.log(c.value);
        // Handle obvious Invalid Date
        if (c.value.toString() === 'Invalid Date') {
            isValid = false;
            // Handle Moment Date
        }
        else if (typeof c.value._isValid === 'boolean') {
            isValid = c.value._isValid;
            // Handle Native Date
        }
        else {
            isValid = !isNaN(new Date(c.value).getTime());
        }
        if (isValid || c.value === '') {
            return null;
        }
        else {
            return {
                date: {
                    valid: false
                }
            };
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    static requiredCheck(c) {
        if (c.value) {
            return null;
        }
        else {
            return {
                required: {
                    valid: false
                }
            };
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    static unique(c) {
        if (!c.value || !Array.isArray(c.value)) {
            return;
        }
        /** @type {?} */
        const testArray = c.value.filter(Boolean);
        /** @type {?} */
        const isValid = testArray.length === Array.from(new Set(testArray)).length;
        if (isValid) {
            return null;
        }
        else {
            return {
                unique: {
                    valid: false
                }
            };
        }
    }
}
AnjunaValidators.minDate = (min) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value < new Date(min)) {
            return {
                min: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};
AnjunaValidators.maxDate = (max) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value > new Date(max)) {
            return {
                max: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class DateValidatorDirective {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return AnjunaValidators.date(c);
    }
}
DateValidatorDirective.decorators = [
    { type: Directive, args: [{
                selector: '[date][ngModel],[date][formControl]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: DateValidatorDirective, multi: true }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DatePickerDirective {
    /**
     * @param {?} renderer
     * @param {?} _element
     * @param {?} changeRef
     * @param {?} dateAdapter
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?=} ngControl
     * @param {?=} _locale
     */
    constructor(renderer, _element, changeRef, dateAdapter, parentForm, parentFormGroup, ngControl, _locale) {
        this.renderer = renderer;
        this._element = _element;
        this.changeRef = changeRef;
        this.dateAdapter = dateAdapter;
        this.ngControl = ngControl;
        this.onChange = () => {
        };
        this.onTouch = () => {
        };
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
        this.valueChange = new EventEmitter();
        this.adapterStrategy = dateAdapter;
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
        this.parent = parentForm ? parentForm : parentFormGroup ? parentFormGroup : null;
    }
    // Custom input to return inner value, but display masked value
    /**
     * @return {?}
     */
    get value() { return this.innerDateValue; }
    /**
     * @param {?} dateValue
     * @return {?}
     */
    set value(dateValue) {
        if (dateValue) {
            this.innerDateValue = this.adapterStrategy.processDateInput(dateValue);
            this.outerValue = this.adapterStrategy.prepDateStringOutput(dateValue);
            this.changeRef.markForCheck();
        }
    }
    // AdapterStrategy will be set by the DateAdapter injected provider. This is for an optional override.
    /**
     * @param {?} adapter
     * @return {?}
     */
    set adapter(adapter) {
        this.adapterStrategy = adapter;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.ngControl) {
            this.value = this.ngControl.value;
            this.changeRef.detectChanges();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.listenerFn) {
            this.listenerFn();
        }
    }
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.value = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.onTouch();
            this.updateInnerValue(event.target.value);
            this.valueChange.emit(this.innerDateValue);
            this.isInvalid();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onCalendarSelect(event) {
        this.onTouch();
        this.updateInnerValue(event.detail);
        this.valueChange.emit(this.innerDateValue);
        this.isInvalid();
    }
    /**
     * @private
     * @return {?}
     */
    isInvalid() {
        if (!this.ngControl) {
            return;
        }
        /** @type {?} */
        const isSubmitted = (this.parent && this.parent.submitted);
        this.ngInvalid = this.ngControl.invalid && (this.ngControl.touched || isSubmitted);
        this._element.nativeElement
            .dispatchEvent(new CustomEvent('anjInvalid', {
            detail: this.ngInvalid,
            bubbles: true
        }));
        this.changeRef.markForCheck();
    }
    // convert the masked value to storage value
    /**
     * @private
     * @param {?} inputDate
     * @return {?}
     */
    updateInnerValue(inputDate) {
        this.innerDateValue = this.adapterStrategy.processDateInput(inputDate);
        this.propogateInnerValueChange();
    }
    /**
     * @private
     * @return {?}
     */
    propogateInnerValueChange() {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
        this.valueChange.emit(this.innerDateValue);
    }
}
DatePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjDatePicker]',
                exportAs: 'anjDatepickerInput'
            },] }
];
/** @nocollapse */
DatePickerDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: DateAdapter },
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
];
DatePickerDirective.propDecorators = {
    value: [{ type: Input }],
    adapter: [{ type: Input }],
    outerValue: [{ type: HostBinding, args: ['value',] }],
    invalid: [{ type: HostBinding, args: ['attr.invalid',] }],
    languageCode: [{ type: HostBinding, args: ['attr.language',] }],
    onInput: [{ type: HostListener, args: ['anjBlur', ['$event'],] }],
    onCalendarSelect: [{ type: HostListener, args: ['anjCalendarSelection', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class TextValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     */
    constructor(parentForm, parentFormGroup, element) {
        super(element, parentForm, parentFormGroup);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onAnjChange(value) {
        this.onChangeEvent(value);
        if (this.element.nativeElement.invalid) {
            this.checkValidity();
        }
    }
}
TextValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input, anj-textarea',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: TextValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
TextValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef }
];
TextValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.detail.value'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class RequiredCheckDirective {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return AnjunaValidators.requiredCheck(c);
    }
}
RequiredCheckDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-checkbox[required][ngModel], anj-toggle[required][ngModel]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: RequiredCheckDirective, multi: true }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class SelectValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     */
    constructor(parentForm, parentFormGroup, element) {
        super(element, parentForm, parentFormGroup);
    }
    /**
     * @param {?} target
     * @return {?}
     */
    onAnjChange(target) {
        // ignore the input for type-aheads
        if (target.matches('anj-input')) {
            return;
        }
        this.onChangeEvent(target.value);
        this.checkValidity();
    }
}
SelectValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-radio-group, anj-select, anj-type-ahead',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: SelectValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
SelectValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef }
];
SelectValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TimePickerDirective {
    /**
     * @param {?} _element
     * @param {?=} ngControl
     * @param {?=} _locale
     * @param {?=} _timezone
     * @param {?=} _meridiem
     */
    constructor(_element, ngControl, _locale, _timezone, _meridiem) {
        this._element = _element;
        this.ngControl = ngControl;
        this._locale = _locale;
        this._timezone = _timezone;
        this._meridiem = _meridiem;
        this.onChange = () => {
        };
        this.onTouch = () => {
        };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        /** @type {?} */
        const picker = this._element.nativeElement.children[0];
        picker.setAttribute('locale', this._locale);
        picker.setAttribute('meridiem', this._meridiem);
        picker.setAttribute('timezone', this._timezone);
    }
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.value = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        this.value = event.detail;
        if (this.ngControl) {
            this.ngControl.control.setValue(this.value);
        }
        this.valueChange.emit(this.value);
    }
}
TimePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjTimePicker]',
                exportAs: 'anjTimepickerInput'
            },] }
];
/** @nocollapse */
TimePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TENANT_TIMEZONE_STRING,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TENANT_MERIDIEM,] }] }
];
TimePickerDirective.propDecorators = {
    value: [{ type: HostBinding, args: ['value',] }],
    onInput: [{ type: HostListener, args: ['anjTimeChanged', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ZuoraDatePipe {
    /**
     * @param {?} datePipe
     */
    constructor(datePipe) {
        this.datePipe = datePipe;
    }
    /**
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    transform(value, args) {
        try {
            return this.datePipe.transform(value, 'shortDate');
        }
        catch (e) {
            return;
        }
    }
}
ZuoraDatePipe.decorators = [
    { type: Pipe, args: [{
                name: 'zuoraDate'
            },] }
];
/** @nocollapse */
ZuoraDatePipe.ctorParameters = () => [
    { type: DatePipe }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ToastService {
    constructor() {
        this.controller = document.querySelector('anj-toast-controller') || ToastService.createController();
    }
    /**
     * @return {?}
     */
    static createController() {
        /** @type {?} */
        const controller = document.createElement('anj-toast-controller');
        document.body.appendChild(controller);
        return (/** @type {?} */ (controller));
    }
    /**
     * @return {?}
     */
    dismissAll() {
        this.controller.dismissAll();
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    error(message, header) {
        return this.controller.error(message, header);
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    message(message, header) {
        return this.controller.message(message, header);
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    success(message, header) {
        return this.controller.success(message, header);
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    warn(message, header) {
        return this.controller.warn(message, header);
    }
}
ToastService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
ToastService.ctorParameters = () => [];
/** @nocollapse */ ToastService.ngInjectableDef = defineInjectable({ factory: function ToastService_Factory() { return new ToastService(); }, token: ToastService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LocalStorageService extends LocalStorage {
}
LocalStorageService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ LocalStorageService.ngInjectableDef = defineInjectable({ factory: function LocalStorageService_Factory() { return new LocalStorageService(); }, token: LocalStorageService, providedIn: "root" });
class SessionStorageService extends SessionStorage {
}
SessionStorageService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ SessionStorageService.ngInjectableDef = defineInjectable({ factory: function SessionStorageService_Factory() { return new SessionStorageService(); }, token: SessionStorageService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DECLARATIONS = [
    // Common
    DraggableDirective,
    DroppableDirective,
    ResizerDirective,
    TemplateDirective,
    TransitionDirective,
    ZuoraDatePipe,
    // Forms
    BooleanValueDirective,
    DatePickerDirective,
    DateValidatorDirective,
    RequiredCheckDirective,
    SelectValueDirective,
    TextValueDirective,
    TimePickerDirective,
    // Tables
    AnjTable
];
class AnjunaCoreModule {
}
AnjunaCoreModule.decorators = [
    { type: NgModule, args: [{
                declarations: DECLARATIONS,
                exports: DECLARATIONS,
                providers: [
                    DatePipe,
                    {
                        provide: DateAdapter,
                        useClass: NativeDateAdapter,
                        deps: [LOCALE_ID]
                    },
                    ZuoraDatePipe
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { getMomentDateAdapter, MomentDateAdapter, UTCDateAdapter, DraggableDirective, DroppableDirective, ResizerDirective, AnjTable, TemplateDirective, TransitionDirective, BooleanValueDirective, DateValidatorDirective, DatePickerDirective, TextValueDirective, RequiredCheckDirective, SelectValueDirective, TimePickerDirective, TENANT_TIMEZONE_STRING, TENANT_MERIDIEM, TIMEZONE_OVERRIDE, MOMENT, AnjunaValidators, ZuoraDatePipe, ToastService, LocalStorageService, SessionStorageService, getWeekdays, getMonths, localize, parseInput, STANDARD_LOCALES, DateAdapter, NativeDateAdapter, TransitionState, AnjunaCoreModule, ValueAccessor as ɵa };

//# sourceMappingURL=anjuna-angular-core.js.map