import { DateSegments } from '../interfaces/DateSegments';
import { DayWithMetadata } from '../interfaces/DayWithMetadata';
import { DateRangeValidationParams } from '../interfaces/DateRangeValidationParams';
export declare const STANDARD_LOCALES: {
    [key: string]: string;
};
export declare abstract class DateAdapter<T> {
    abstract dayOfFirst(year: number, month: number): number;
    abstract daysInMonth(year: number, month: number): number;
    abstract getMonthInfo(year: number, month: number): T;
    abstract isDisabled(evalDate: Date, dateRangeParams: DateRangeValidationParams): boolean;
    abstract month(date: Date): number;
    abstract newDateFromArray(year?: number, month?: number, day?: number): T;
    abstract newDateFromValue(value?: any): T;
    abstract prepHeaderDate(date: Date, weekDays?: string[], months?: string[]): string;
    abstract prepHeaderYear(date: Date, weekDays?: string[], months?: string[]): string;
    abstract sameDay(_calendarDay: DayWithMetadata, _comparator: Date): boolean;
    abstract year(date: Date): number;
    abstract weekdays(): string[];
    abstract months(): string[];
    abstract prepDateStringOutput(input: T): string;
    abstract processDateInput(input: T | string): Date;
}
export declare class NativeDateAdapter implements DateAdapter<Date> {
    private _locale?;
    private languageCode;
    constructor(_locale?: string);
    dayOfFirst(year: number, month: number): number;
    daysInMonth(year: number, month: number): number;
    isDisabled(evalDate: Date, dateRangeParams: DateRangeValidationParams): boolean;
    getMonthInfo(year: number, month: number): Date;
    month(date: Date): number;
    newDateFromArray(year?: number, month?: number, day?: number): Date;
    newDateFromValue(value?: any): Date;
    prepHeaderDate(date: Date): string;
    prepHeaderYear(date: Date): string;
    sameDay(_calendarDay: DayWithMetadata, _comparator: Date): boolean;
    year(date: Date): number;
    weekdays(): string[];
    months(): string[];
    prepDateStringOutput(input: Date): string;
    processDateInput(input: Date | string): Date;
}
export declare function getWeekdays(lang: string): string[];
export declare function getMonths(lang: string): string[];
export declare function localize(_languageCode: string, date: Date, format: string): string;
export declare function parseInput(input: string, locale: string): DateSegments;
